import requests
import json
import shutil
import pandas as pd
import re
import os.path

ENDPOINT = "https://graphql.dev.mint.isi.edu/v1/graphql"
SECRET = "secret"

def post_query(query, variables = {}):
    response = requests.post(ENDPOINT, json = {
        "query": query,
        "variables": variables
    }, headers = {"X-Hasura-Admin-Secret": SECRET})
    return response.json()

def get_thread_run_details(threadId):
    query = """
    query executions_for_thread_model($threadId: String!) {
      execution(where: {thread_model_executions: {thread_model: { thread_id: {_eq: $threadId}}}}) {
        id
        parameter_bindings {
          model_parameter {
            name
          }
          parameter_value
        }
        data_bindings {
          model_io {
            name
          }
          resource {
            id
            name
            url
            spatial_coverage
          }
        }
        results {
          model_output {
            name
          }
          resource {
            id
            name
            url
          }
        }
      }
    }
    """
    
    executions = []
    
    response = post_query(query, {"threadId": threadId})
    for ex in response["data"]["execution"]:
        execution = {
            "id": ex["id"],
            "location": {
                "lat": None,
                "lon": None
            },
            "parameters": {},
            "inputs": {},
            "outputs": {}
        }
        for p in ex["parameter_bindings"]:
            execution["parameters"][p["model_parameter"]["name"]] = p["parameter_value"]
        for d in ex["data_bindings"]:
            execution["inputs"][d["model_io"]["name"]] = d["resource"]["url"]
            if d["resource"]["spatial_coverage"] is not None:
                coords = d["resource"]["spatial_coverage"]["coordinates"]
                execution["location"]["lat"] = coords[1]
                execution["location"]["lon"] = coords[0]
        for d in ex["results"]:
            execution["outputs"][d["model_output"]["name"]] = d["resource"]["url"]
        executions.append(execution)
        
    return executions


def copy_file(url):
    srcfile = re.sub(r".+/(local|tacc)-execution/", r"\1-execution/", url)
    destfile = re.sub(r".+/", "cycles-output-data/", url)
    if not os.path.exists(destfile):
        shutil.copyfile(srcfile, destfile)

#executions = get_thread_run_details("3xTiJ0FuEw6xhRZ8JpVk") #Maize Ethiopia
executions = get_thread_run_details("SuSnJYaGusaUtMikTjFj") #Sorghum
#executions = get_thread_run_details("KzML3JBu6wzGFoYTQabu") #Cassava
#executions = get_thread_run_details("F44zqJIFP7SUc5Y9W0RL") #Potato
#executions = get_thread_run_details("5qSgNtqqUSakF0zVryiJ") #Millet
#executions = get_thread_run_details("ysCyagpUlBLtRX2vunsV") #SpringLentil

for ex in executions:
    try:
        copy_file(ex["outputs"]["cycles_season"])
        copy_file(ex["outputs"]["cycles_summary"])
    except:
        print(ex["id"])
